@@ -1,3 +1,355 @@
+#' Network meta-analysis of binary outcome data
+#' 
+#' @description
+#' Provides three models for the network meta-analysis of binary data
+#' (Mantel-Haenszel method, based on the non-central hypergeometric
+#' distribution, and the inverse variance method).
+#' 
+#' @param event1 Number of events (first treatment).
+#' @param n1 Number of observations (first treatment).
+#' @param event2 Number of events (second treatment).
+#' @param n2 Number of observations (second treatment)
+#' @param treat1 Label/Number for first treatment.
+#' @param treat2 Label/Number for second treatment.
+#' @param studlab An optional - but important! - vector with study
+#'   labels (see Details).
+#' @param data An optional data frame containing the study
+#'   information.
+#' @param subset An optional vector specifying a subset of studies to
+#'   be used.
+#' @param sm A character string indicating underlying summary measure,
+#'   i.e., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"}.
+#' @param method A character string indicating which method is to be
+#'   used for pooling of studies. One of \code{"Inverse"},
+#'   \code{"MH"}, or \code{"NCH"}, can be abbreviated.
+#' @param cc.pooled A logical indicating whether \code{incr} should be
+#'   used as a continuity correction, when calculating the network
+#'   meta-analysis estimates.
+#' @param incr A numerical value which is added to each cell count,
+#'   i.e., to the numbers of events and non-events, of all treatment
+#'   arms in studies with zero events or non-events in any of the
+#'   treatment arms ("continuity correction").
+#' @param allincr A logical indicating whether \code{incr} should be
+#'   added to each cell count of all studies if a continuity
+#'   correction was used for at least one study (only considered if
+#'   \code{method = "Inverse"}). If FALSE (default), \code{incr} is
+#'   used as continuity correction only for studies with zero events
+#'   or zero non-events in any of the treatment arms.
+#' @param addincr A logical indicating whether \code{incr} should be
+#'   added to each cell count of all studies, irrespective of zero
+#'   cell counts (only considered if \code{method = "Inverse"}).
+#' @param allstudies A logical indicating whether studies with zero
+#'   events or non-events in all treatment arms should be included in
+#'   an inverse variance meta-analysis (applies only if \code{method =
+#'   "Inverse"} and \code{sm} is equal to either \code{"RR"} or
+#'   \code{"OR"}).
+#' @param level The level used to calculate confidence intervals for
+#'   individual studies.
+#' @param level.comb The level used to calculate confidence intervals
+#'   for pooled estimates.
+#' @param comb.fixed A logical indicating whether a fixed effects
+#'   (common effects) network meta-analysis should be conducted.
+#' @param comb.random A logical indicating whether a random effects
+#'   network meta-analysis should be conducted.
+#' @param prediction A logical indicating whether a prediction
+#'   interval should be printed (only considered if \code{method =
+#'   "Inverse"}).
+#' @param level.predict The level used to calculate prediction
+#'   interval for a new study (only considered if \code{method =
+#'   "Inverse"}).
+#' @param reference.group Reference treatment.
+#' @param baseline.reference A logical indicating whether results
+#'   should be expressed as comparisons of other treatments versus the
+#'   reference treatment (default) or vice versa. This argument is
+#'   only considered if \code{reference.group} has been specified.
+#' @param all.treatments A logical or \code{"NULL"}. If \code{TRUE},
+#'   matrices with all treatment effects, and confidence limits will
+#'   be printed.
+#' @param seq A character or numerical vector specifying the sequence
+#'   of treatments in printouts.
+#' @param tau.preset An optional value for manually setting the
+#'   square-root of the between-study variance \eqn{\tau^2} (only
+#'   considered if \code{method = "Inverse"}).
+#' @param tol.multiarm A numeric for the tolerance for consistency of
+#'   treatment estimates and corresponding variances in multi-arm
+#'   studies which are consistent by design (only considered if
+#'   \code{method = "Inverse"}).
+#' @param details.chkmultiarm A logical indicating whether treatment
+#'   estimates and / or variances of multi-arm studies with
+#'   inconsistent results or negative multi-arm variances should be
+#'   printed (only considered if \code{method = "Inverse"}).
+#' @param sep.trts A character used in comparison names as separator
+#'   between treatment labels.
+#' @param nchar.trts A numeric defining the minimum number of
+#'   characters used to create unique treatment names (see Details).
+#' @param backtransf A logical indicating whether results should be
+#'   back transformed in printouts and forest plots. If
+#'   \code{backtransf = TRUE}, results for \code{sm = "OR"} are
+#'   presented as odds ratios rather than log odds ratios, for
+#'   example.
+#' @param title Title of meta-analysis / systematic review.
+#' @param keepdata A logical indicating whether original data (set)
+#'   should be kept in netmeta object.
+#' @param warn A logical indicating whether warnings should be printed
+#'   (e.g., if studies are excluded from meta-analysis due to zero
+#'   standard errors).
+#' 
+#' @details
+#' This function implements three models for the network meta-analysis
+#' of binary data:
+#' \itemize{
+#' \item The Mantel-Haenszel network meta-analysis model, as described
+#'   in Efthimiou et al. (2019) (\code{method = "MH"});
+#' \item a network meta-analysis model using the non-central
+#'   hypergeometric distribution with the Breslow approximation, as
+#'   described in Stijnen et al. (2010) (\code{method = "NCH"});
+#' \item the inverse variance method for network meta-analysis
+#'   (\code{method = "Inverse"}), also provided by
+#'   \code{\link{netmeta}}.
+#' }
+#' 
+#' Comparisons belonging to multi-arm studies are identified by
+#' identical study labels (argument \code{studlab}). It is therefore
+#' important to use identical study labels for all comparisons
+#' belonging to the same multi-arm study.
+#' 
+#' Data entry for this function is in \emph{contrast-based} format,
+#' that is, each line of the data corresponds to a single pairwise
+#' comparison between two treatments (arguments \code{treat1},
+#' \code{treat2}, \code{event1}, \code{n1}, \code{event2}, and
+#' \code{n2}). If data are provided in \emph{arm-based} format, that
+#' is, number of events and participants are given for each treatment
+#' arm separately, function \code{\link{pairwise}} can be used to
+#' transform the data to \emph{contrast-based} format (see help page
+#' of function \code{\link{pairwise}}).
+#' 
+#' Note, all pairwise comparisons must be provided for a multi-arm
+#' study. Consider a multi-arm study of \emph{p} treatments with known
+#' variances. For this study, the number of events and observations
+#' must be provided for each treatment, for each of \emph{p}(\emph{p}
+#' - 1) / 2 possible comparisons in separate lines in the data. For
+#' instance, a three-arm study contributes three pairwise comparisons,
+#' a four-arm study even six pairwise comparisons. Function
+#' \code{\link{pairwise}} automatically calculates all pairwise
+#' comparisons for multi-arm studies.
+#' 
+#' For \code{method = "Inverse"}, both fixed effects and random
+#' effects models are calculated regardless of values choosen for
+#' arguments \code{comb.fixed} and \code{comb.random}. Accordingly,
+#' the network estimates for the random effects model can be extracted
+#' from component \code{TE.random} of an object of class
+#' \code{"netmeta"} even if argument \code{comb.random = FALSE}.
+#' However, all functions in R package \bold{netmeta} will adequately
+#' consider the values for \code{comb.fixed} and
+#' \code{comb.random}. E.g. function
+#' \code{\link{print.summary.netmeta}} will not print results for the
+#' random effects model if \code{comb.random = FALSE}.
+#' 
+#' For the random-effects model, the direct treatment estimates are
+#' based on the common between-study variance \eqn{\tau^2} from the
+#' network meta-analysis.
+#'
+#' For \code{method = "MH"} and \code{method = "NCH"}, only a fixed
+#' effects model is available.
+#' 
+#' By default, treatment names are not abbreviated in
+#' printouts. However, in order to get more concise printouts,
+#' argument \code{nchar.trts} can be used to define the minimum number
+#' of characters for abbreviated treatment names (see
+#' \code{\link{abbreviate}}, argument \code{minlength}). R function
+#' \code{\link{treats}} is utilised internally to create abbreviated
+#' treatment names.
+#' 
+#' Names of treatment comparisons are created by concatenating
+#' treatment labels of pairwise comparisons using \code{sep.trts} as
+#' separator (see \code{\link{paste}}). These comparison names are
+#' used in the covariance matrices \code{Cov.fixed} and
+#' \code{Cov.random} and in some R functions, e.g,
+#' \code{\link{decomp.design}}. By default, a colon is used as the
+#' separator. If any treatment label contains a colon the following
+#' characters are used as separator (in consecutive order):
+#' \code{"-"}, \code{"_"}, \code{"/"}, \code{"+"}, \code{"."},
+#' \code{"|"}, and \code{"*"}. If all of these characters are used in
+#' treatment labels, a corresponding error message is printed asking
+#' the user to specify a different separator.
+#'
+#' @return
+#' An object of class \code{netmetabin} and \code{netmeta} with
+#' corresponding \code{print}, \code{summary}, \code{forest}, and
+#' \code{netrank} functions. The object is a list containing the
+#' following components:
+#' \item{studlab, treat1, treat2}{As defined above.}
+#' \item{n1, n2, event1, event2}{As defined above.}
+#' \item{TE}{Estimate of treatment effect, i.e. difference between
+#'   first and second treatment (e.g. log odds ratio).}
+#' \item{seTE}{Standard error of treatment estimate.}
+#' \item{k}{Total number of studies.}
+#' \item{m}{Total number of pairwise comparisons.}
+#' \item{n}{Total number of treatments.}
+#' \item{d}{Total number of designs (corresponding to the unique set
+#'   of treatments compared within studies).}
+#' \item{trts}{Treatments included in network meta-analysis.}
+#' \item{k.trts}{Number of studies evaluating a treatment.}
+#' \item{n.trts}{Number of observations receiving a treatment.}
+#' \item{events.trts}{Number of events observed for a treatment.}
+#' \item{studies}{Study labels coerced into a factor with its levels
+#'   sorted alphabetically.}
+#' \item{narms}{Number of arms for each study.}
+#' \item{designs}{Unique list of designs present in the network. A
+#'   design corresponds to the set of treatments compared within a
+#'   study.}
+#' \item{TE.fixed, seTE.fixed}{\emph{n}x\emph{n} matrix with estimated
+#'   overall treatment effects and standard errors for fixed effects
+#'   model.}
+#' \item{lower.fixed, upper.fixed}{\emph{n}x\emph{n} matrices with
+#'   lower and upper confidence interval limits for fixed effects
+#'   model.}
+#' \item{zval.fixed, pval.fixed}{\emph{n}x\emph{n} matrices with
+#'   z-value and p-value for test of overall treatment effect under
+#'   fixed effects model.}
+#' \item{TE.random, seTE.random}{\emph{n}x\emph{n} matrix with
+#'   estimated overall treatment effects and standard errors for
+#'   random effects model (only available if \code{method =
+#'   "Inverse"}).}
+#' \item{lower.random, upper.random}{\emph{n}x\emph{n} matrices with
+#'   lower and upper confidence interval limits for random effects
+#'   model (only available if \code{method = "Inverse"}).}
+#' \item{zval.random, pval.random}{\emph{n}x\emph{n} matrices with
+#'   z-value and p-value for test of overall treatment effect under
+#'   random effects model (only available if \code{method =
+#'   "Inverse"}).}
+#' \item{TE.direct.fixed, seTE.direct.fixed}{\emph{n}x\emph{n} matrix
+#'   with estimated treatment effects and standard errors from direct
+#'   evidence under fixed effects model.}
+#' \item{lower.direct.fixed, upper.direct.fixed}{\emph{n}x\emph{n}
+#'   matrices with lower and upper confidence interval limits from
+#'   direct evidence under fixed effects model.}
+#' \item{zval.direct.fixed, pval.direct.fixed}{\emph{n}x\emph{n}
+#'   matrices with z-value and p-value for test of overall treatment
+#'   effect from direct evidence under fixed effects model.}
+#' \item{TE.direct.random, seTE.direct.random}{\emph{n}x\emph{n}
+#'   matrix with estimated treatment effects and standard errors from
+#'   direct evidence under random effects model (only available if
+#'   \code{method = "Inverse"}).}
+#' \item{lower.direct.random, upper.direct.random}{\emph{n}x\emph{n}
+#'   matrices with lower and upper confidence interval limits from
+#'   direct evidence under random effects model (only available if
+#'   \code{method = "Inverse"}).}
+#' \item{zval.direct.random, pval.direct.random}{\emph{n}x\emph{n}
+#'   matrices with z-value and p-value for test of overall treatment
+#'   effect from direct evidence under random effects model (only
+#'   available if \code{method = "Inverse"}).}
+#' \item{Q}{Overall heterogeneity / inconsistency statistic.}
+#' \item{df.Q}{Degrees of freedom for test of heterogeneity /
+#'   inconsistency.}
+#' \item{pval.Q}{P-value for test of heterogeneity / inconsistency.}
+#' \item{I2}{I-squared (only available if \code{method = "Inverse"}).}
+#' \item{tau}{Square-root of between-study variance (only available if
+#'   \code{method = "Inverse"}).}
+#' \item{A.matrix}{Adjacency matrix (\emph{n}x\emph{n}).}
+#' \item{H.matrix}{Hat matrix (\emph{m}x\emph{m})}
+#' \item{n.matrix}{\emph{n}x\emph{n} matrix with number of
+#'   observations in direct comparisons.}
+#' \item{events.matrix}{\emph{n}x\emph{n} matrix with number of events
+#'   in direct comparisons.}
+#' \item{sm, method, level, level.comb}{As defined above.}
+#' \item{incr, allincr, addincr, allstudies, cc.pooled}{As defined
+#'   above.}
+#' \item{comb.fixed, comb.random}{As defined above.}
+#' \item{prediction, level.predict}{As defined above.}
+#' \item{reference.group, baseline.reference, all.treatments}{As
+#'   defined above.}
+#' \item{seq, tau.preset, tol.multiarm, details.chkmultiarm}{As
+#'   defined above.}
+#' \item{sep.trts, nchar.trts}{As defined above.}
+#' \item{backtransf, title, warn}{As defined above.}
+#' \item{data}{Data set (in contrast-based format).}
+#' \item{data.design}{List with data in arm-based format (each list
+#'   element corresponds to a single design).}
+#' \item{call}{Function call.}
+#' \item{version}{Version of R package netmeta used to create object.}
+#' 
+#' @author Orestis Efthimiou \email{oremiou@@gmail.com}, Guido
+#'   Schwarzer \email{sc@@imbi.uni-freiburg.de}
+#' 
+#' @seealso \code{\link{pairwise}}, \code{\link{netmeta}}
+#' 
+#' @references
+#' Efthimiou O, Rücker G, Schwarzer G, Higgins J, Egger M, Salanti G
+#' (2019):
+#' A Mantel-Haenszel model for network meta-analysis of rare events.
+#' \emph{Statistics in Medicine},
+#' 1--21, https://doi.org/10.1002/sim.8158
+#' 
+#' Senn S, Gavini F, Magrez D, Scheen A (2013):
+#' Issues in performing a network meta-analysis.
+#' \emph{Statistical Methods in Medical Research},
+#' \bold{22}, 169--89
+#' 
+#' Stijnen T, Hamza TH, Ozdemir P (2010):
+#' Random effects meta-analysis of event outcome in the framework of
+#' the generalized linear mixed model with applications in sparse
+#' data.
+#' \emph{Statistics in Medicine},
+#' \bold{29}, 3046--67
+
+#' @examples
+#' data(Dong2013)
+#' 
+#' # Only consider first ten studies (to reduce runtime of example)
+#' #
+#' first10 <- subset(Dong2013, id <= 10)
+#' 
+#' # Transform data from long arm-based format to contrast-based
+#' # format. Argument 'sm' has to be used for odds ratio as summary
+#' # measure; by default the risk ratio is used in the metabin
+#' # function called internally.
+#' #
+#' p1 <- pairwise(treatment, death, randomized, studlab = id,
+#'                data = first10, sm = "OR")
+#' 
+#' # Conduct Mantel-Haenszel network meta-analysis (without continuity
+#' # correction)
+#' #
+#' nb1 <- netmetabin(p1, ref = "plac")
+#' nb1
+#' 
+#' # Obtain the league table
+#' #
+#' netleague(nb1)
+#' 
+#' \dontrun{
+#' # Conduct Mantel-Haenszel network meta-analysis for the whole
+#' # dataset
+#' #
+#' p2 <- pairwise(treatment, death, randomized, studlab = id,
+#'                data = Dong2013, sm = "OR")
+#' netmetabin(p2, ref = "plac")
+#'   
+#' # Conduct network meta-analysis using the non-central
+#' # hypergeometric model (without continuity correction)
+#' #
+#' netmetabin(p2, ref = "plac", method = "NCH")
+#' 
+#' # Conduct Mantel-Haenszel network meta-analysis (with continuity
+#' # correction of 0.5; include all studies)
+#' #
+#' netmetabin(p2, ref = "plac", cc.pooled = TRUE)
+#' 
+#' data(Gurusamy2011)
+#' 
+#' p3 <- pairwise(treatment, death, n, studlab = study,
+#'                data = Gurusamy2011, sm = "OR")
+#' 
+#' # Conduct Mantel-Haenszel network meta-analysis (without continuity
+#' # correction)
+#' #
+#' netmetabin(p3, ref = "cont")
+#' }
+#' 
+#' @export netmetabin
+
+
 netmetabin <- function(event1, n1, event2, n2,
                        treat1, treat2, studlab,
                        data = NULL, subset = NULL,
@@ -216,37 +568,43 @@ netmetabin <- function(event1, n1, event2, n2,
   ##
   ##
   ## (2b) Store complete dataset in list object data
-  ##      (if argument keepdata is TRUE)
   ##
   ##
-  if (keepdata) {
-    if (nulldata & !is.pairwise)
-      data <- data.frame(.event1 = event1)
-    else if (nulldata & is.pairwise) {
-      data <- pairdata
-      data$.order <- .order
-      data$.event1 <- event1
-    }
-    else
-      data$.event1 <- event1
-    ##
-    data$.n1 <- n1
-    data$.event2 <- event2
-    data$.n2 <- n2
-    data$.treat1 <- treat1
-    data$.treat2 <- treat2
-    data$.studlab <- studlab
+  if (nulldata & !is.pairwise)
+    data <- data.frame(.event1 = event1)
+  else if (nulldata & is.pairwise) {
+    data <- pairdata
     data$.order <- .order
-    ##
-    if (!missing.subset) {
-      if (length(subset) == dim(data)[1])
-        data$.subset <- subset
-      else {
-        data$.subset <- FALSE
-        data$.subset[subset] <- TRUE
-      }
+    data$.event1 <- event1
+  }
+  else
+    data$.event1 <- event1
+  ##
+  data$.n1 <- n1
+  data$.event2 <- event2
+  data$.n2 <- n2
+  data$.treat1 <- treat1
+  data$.treat2 <- treat2
+  data$.studlab <- studlab
+  data$.order <- .order
+  ##
+  if (!missing.subset) {
+    if (length(subset) == dim(data)[1])
+      data$.subset <- subset
+    else {
+      data$.subset <- FALSE
+      data$.subset[subset] <- TRUE
     }
   }
+  ##
+  m.data <- metabin(event1, n1, event2, n2,
+                    sm = sm, method = "Inverse",
+                    incr = incr, allincr = allincr,
+                    addincr = addincr, allstudies = allstudies,
+                    warn = FALSE)
+  ##
+  data$.TE <- m.data$TE
+  data$.seTE <- m.data$seTE
   
   
   ##
@@ -452,20 +810,6 @@ netmetabin <- function(event1, n1, event2, n2,
     res
   }
   ##
-  set.designs <- function(x, tdat.design, all.designs) {
-    x$design <- NULL
-    x <- merge(x, tdat.design, by = "studlab", all = TRUE)
-    x$design <- as.character(x$design)
-    ##
-    if (any(is.na(x$design))) {
-      sel.studlab <- unique(x$studlab[is.na(x$design)])
-      for (i in sel.studlab)
-        x$design[x$studlab == i] <-
-          all.designs$design[all.designs$studlab == i]
-    }
-    x
-  }
-  ##
   data$.drop <- rep(FALSE, nrow(data))
   ##
   ## Add variable 'non.event'
@@ -498,7 +842,9 @@ netmetabin <- function(event1, n1, event2, n2,
   ##
   names(tdat.design) <- c("studlab", ".design")
   ##
-  data <- merge(data, tdat.design, by = "studlab", all.x = TRUE)
+  data <- merge(data, tdat.design,
+                by.x = ".studlab", by.y = "studlab",
+                all.x = TRUE)
   data <- data[order(data$.order), ]
   ##
   rm(tdat.design)
@@ -506,40 +852,55 @@ netmetabin <- function(event1, n1, event2, n2,
   
   ##
   ##
-  ## (6) Stage I: setting up the data (Efthimiou et al., 2018)
+  ## (6) Stage I: setting up the data (Efthimiou et al., 2019)
   ##
   ##
-  ## Step i. Remove all-zero studies (only MH and NCH methods)
+  ## Step i. Remove all-zero or all-event studies (only MH and NCH methods)
   ##
   if (method != "Inverse") {
-    n.events <- with(dat.long, tapply(event, studlab, sum))
+    events.study <- with(dat.long, tapply(event, studlab, sum))
+    nonevents.study <- with(dat.long, tapply(n - event, studlab, sum))
     ##
-    if (any(n.events == 0)) {
-      allzero <- n.events == 0
+    if (any(events.study == 0) | any(nonevents.study == 0)) {
+      zeroevents <- events.study == 0
+      allevents <- nonevents.study == 0
+      keep <- !(zeroevents | allevents)
       ##
-      if (warn)
-        if (sum(allzero) == 1)
-          warning("Study '", names(n.events)[allzero],
+      if (warn) {
+        if (sum(zeroevents) == 1)
+          warning("Study '", names(events.study)[zeroevents],
                   "' without any events excluded from network meta-analysis.",
                   call. = FALSE)
-        else
+        else if (sum(zeroevents) > 1)
           warning("Studies without any events excluded ",
                   "from network meta-analysis: ",
-                  paste(paste0("'", names(n.events)[allzero], "'"),
+                  paste(paste0("'", names(events.study)[zeroevents], "'"),
                         collapse = " - "),
                   call. = FALSE)
+        ##
+        if (sum(allevents) == 1)
+          warning("Study '", names(nonevents.study)[allevents],
+                  "' with all events excluded from network meta-analysis.",
+                  call. = FALSE)
+        else if (sum(allevents) > 1)
+          warning("Studies with all events excluded ",
+                  "from network meta-analysis: ",
+                  paste(paste0("'", names(nonevents.study)[allevents], "'"),
+                        collapse = " - "),
+                  call. = FALSE)
+      }
       ##
-      dat.long <- dat.long[dat.long$studlab %in% names(n.events)[!allzero], ,
+      dat.long <- dat.long[dat.long$studlab %in% names(events.study)[keep], ,
                            drop = FALSE]
-      dat.wide <- dat.wide[dat.wide$studlab %in% names(n.events)[!allzero], ,
+      dat.wide <- dat.wide[dat.wide$studlab %in% names(events.study)[keep], ,
                            drop = FALSE]
       ##
-      data$.drop <- data$.drop | data$studlab %in% names(n.events)[allzero]
+      data$.drop <- data$.drop | data$studlab %in% names(events.study)[!keep]
       ##
-      rm(allzero)
+      rm(zeroevents, allevents, keep)
     }
     ##
-    rm(n.events)
+    rm(events.study)
   }
   ##
   ## Add variable 'study' with study numbers
@@ -559,29 +920,38 @@ netmetabin <- function(event1, n1, event2, n2,
   dat.wide$incr <- 0
   data$.incr <- 0
   ##
-  ## Step iii. Drop treatment arms without events from individual
+  ## Step iii. Drop treatment arms without or all events from individual
   ##           designs (argument 'cc.pooled' is FALSE) or add
   ##           increment if argument 'cc.pooled' is TRUE and argument
   ##           'incr' is larger than zero
   ##
   if (method != "Inverse") {
     ##
-    d.events <- with(dat.long, tapply(event, list(design, treat), sum))
+    events.arm <- with(dat.long, tapply(event, list(design, treat), sum))
+    nonevents.arm <- with(dat.long, tapply(n - event, list(design, treat), sum))
     ##
-    if (any(d.events == 0, na.rm = TRUE)) {
-      zero <- d.events == 0
-      zerocells <- as.data.frame(which(zero, arr.ind = TRUE),
+    if (any(events.arm == 0, na.rm = TRUE) |
+        any(nonevents.arm == 0, na.rm = TRUE)) {
+      ##
+      ## Identify entries without events
+      ##
+      zeroevents <- events.arm == 0
+      zerocells <- as.data.frame(which(zeroevents, arr.ind = TRUE),
                                  stringsAsFactors = FALSE)
       ##
-      zerocells$design <- rownames(zero)[zerocells$row]
-      zerocells$treat <- colnames(zero)[zerocells$col]
+      zerocells$design <- rownames(zeroevents)[zerocells$row]
+      zerocells$treat <- colnames(zeroevents)[zerocells$col]
       ##
       zero.long <- rep(0, nrow(dat.long))
       zero.wide <- rep(0, nrow(dat.wide))
       zero.data <- rep(0, nrow(data))
       ##
       for (i in seq_along(zerocells$design)) {
-        zero.long <- zero.long + (dat.long$design == zerocells$design[i])
+        if (!cc.pooled)
+          zero.long <- zero.long + (dat.long$design == zerocells$design[i] &
+                                    dat.long$treat == zerocells$treat[i])
+        else
+          zero.long <- zero.long + (dat.long$design == zerocells$design[i])
         ##
         zero.wide <- zero.wide + (dat.wide$design == zerocells$design[i] &
                                   (dat.wide$treat1 == zerocells$treat[i] |
@@ -596,14 +966,54 @@ netmetabin <- function(event1, n1, event2, n2,
       zero.wide <- zero.wide > 0
       zero.data <- zero.data > 0
       ##
+      ## Identify entries with all events
+      ##
+      allevents <- nonevents.arm == 0
+      allcells <- as.data.frame(which(allevents, arr.ind = TRUE),
+                                 stringsAsFactors = FALSE)
+      ##
+      allcells$design <- rownames(allevents)[allcells$row]
+      allcells$treat <- colnames(allevents)[allcells$col]
+      ##
+      all.long <- rep(0, nrow(dat.long))
+      all.wide <- rep(0, nrow(dat.wide))
+      all.data <- rep(0, nrow(data))
+      ##
+      for (i in seq_along(allcells$design)) {
+        if (!cc.pooled)
+          all.long <- all.long + (dat.long$design == allcells$design[i] &
+                                  dat.long$treat == allcells$treat[i])
+        else
+          all.long <- all.long + (dat.long$design == allcells$design[i])
+        ##
+        all.wide <- all.wide + (dat.wide$design == allcells$design[i] &
+                                (dat.wide$treat1 == allcells$treat[i] |
+                                 dat.wide$treat2 == allcells$treat[i]))
+        ##
+        all.data <- all.data + (data$.design == allcells$design[i] &
+                                (data$.treat1 == allcells$treat[i] |
+                                 data$.treat2 == allcells$treat[i]))
+      }
+      ##
+      all.long <- all.long > 0
+      all.wide <- all.wide > 0
+      all.data <- all.data > 0
+      ##
+      ## Identify entries with sparse binary data (i.e., with zero or
+      ## all events)
+      ##
+      sparse.long <- zero.long | all.long
+      sparse.wide <- zero.wide | all.wide
+      sparse.data <- zero.data | all.data
+      ##
       if (!cc.pooled) {
-        if (warn)
-          if (sum(zero, na.rm = TRUE) == 1)
+        if (warn) {
+          if (sum(zeroevents, na.rm = TRUE) == 1)
             warning("Treatment arm '", zerocells$treat,
                     "' without events in design '",
                     zerocells$design, "' excluded from network meta-analysis.",
                     call. = FALSE)
-          else
+          else if (sum(zeroevents, na.rm = TRUE) > 1)
             warning("Treatment arms without events in a design excluded ",
                     "from network meta-analysis:\n    ",
                     paste0("'",
@@ -612,53 +1022,45 @@ netmetabin <- function(event1, n1, event2, n2,
                            "'",
                            collapse = " - "),
                     call. = FALSE)
+          ##
+          if (sum(allevents, na.rm = TRUE) == 1)
+            warning("Treatment arm '", allcells$treat,
+                    "' with all events in design '",
+                    allcells$design, "' excluded from network meta-analysis.",
+                    call. = FALSE)
+          else if (sum(allevents, na.rm = TRUE) > 1)
+            warning("Treatment arms with all events in a design excluded ",
+                    "from network meta-analysis:\n    ",
+                    paste0("'",
+                           paste0(paste0(allcells$treat, " in "),
+                                  allcells$design),
+                           "'",
+                           collapse = " - "),
+                    call. = FALSE)
+        }
         ##
-        dat.long <- dat.long[!zero.long, , drop = FALSE]
-        dat.wide <- dat.wide[!zero.wide, , drop = FALSE]
-        data$.drop <- data$.drop | zero.data
-        ##
-        rm(zero, zerocells, zero.long, zero.wide, zero.data)
+        dat.long <- dat.long[!sparse.long, , drop = FALSE]
+        dat.wide <- dat.wide[!sparse.wide, , drop = FALSE]
+        data$.drop <- data$.drop | sparse.data
       }
       else {
-        dat.long$incr[zero.long] <- incr
+        dat.long$incr[sparse.long] <- incr
         ##
-        dat.long$event[zero.long] <- dat.long$event[zero.long] + incr
-        dat.long$non.event[zero.long] <- dat.long$non.event[zero.long] + incr
-        dat.long$n[zero.long] <- dat.long$n[zero.long] + 2 * incr
+        dat.long$event[sparse.long] <- dat.long$event[sparse.long] + incr
+        dat.long$non.event[sparse.long] <- dat.long$non.event[sparse.long] + incr
+        dat.long$n[sparse.long] <- dat.long$n[sparse.long] + 2 * incr
         ##
-        dat.wide$incr[zero.wide] <- incr
+        dat.wide$incr[sparse.wide] <- incr
         ##
-        data$.incr[zero.data] <- incr
-        ##
-        rm(zero.long)
+        data$.incr[sparse.data] <- incr
       }
+      ##
+      rm(zeroevents, zerocells, zero.long, zero.wide, zero.data,
+         allevents, allcells, all.long, all.wide, all.data,
+         sparse.long, sparse.wide, sparse.data)
     }
     ##
-    rm(d.events)
-    ##
-    ## (Re)Add variable 'design' with study design (as treatment arms
-    ## may have been dropped)
-    ##
-    dat.wide$design <- NULL
-    data$.design <- NULL
-    ##
-    tdat.design <- get.designs(dat.long)
-    dat.long <- set.designs(dat.long, tdat.design, all.designs)
-    all.designs <- get.designs(dat.long)
-    ##
-    dat.wide <- merge(dat.wide, tdat.design, by = "studlab")
-    ##
-    dat.long <- dat.long[order(dat.long$.order), ]
-    dat.wide <- dat.wide[order(dat.wide$.order), ]
-    ##
-    names(tdat.design) <- c("studlab", ".design")
-    ##
-    data <- merge(data, tdat.design, by = "studlab", all.x = TRUE)
-    data <- data[order(data$.order), ]
-    ##
-    rm(tdat.design)
-    ##
-    dat.long$design <- as.character(dat.long$design)
+    rm(events.arm, nonevents.arm)
   }
   ##
   ## Step iv. Remove designs with single treatment arm from dataset
@@ -692,29 +1094,6 @@ netmetabin <- function(event1, n1, event2, n2,
     }
     ##
     rm(d.single)
-    ##
-    ## (Re)Add variable 'design' with study design (as treatment arms
-    ## may have been dropped)
-    ##
-    dat.wide$design <- NULL
-    data$.design <- NULL
-    ##
-    tdat.design <- get.designs(dat.long)
-    dat.long <- set.designs(dat.long, tdat.design, all.designs)
-    ##
-    dat.wide <- merge(dat.wide, tdat.design, by = "studlab")
-    ##
-    names(tdat.design) <- c("studlab", ".design")
-    ##
-    data <- merge(data, tdat.design, by = "studlab", all.x = TRUE)
-    data <- data[order(data$.order), ]
-    ##
-    rm(tdat.design)
-    ##
-    dat.long <- dat.long[order(dat.long$design, dat.long$studlab,
-                               dat.long$treat), ]
-    dat.wide <- dat.wide[order(dat.wide$design, dat.wide$studlab,
-                               dat.wide$treat1, dat.wide$treat2), ]
   }
   ##
   dat.long <- dat.long[, c("studlab", "treat",
@@ -1377,8 +1756,8 @@ netmetabin <- function(event1, n1, event2, n2,
               treat1 = treat1,
               treat2 = treat2,
               ##
-              TE = data$TE[!data$.drop],
-              seTE = data$seTE[!data$.drop],
+              TE = data$.TE[!data$.drop],
+              seTE = data$.seTE[!data$.drop],
               seTE.adj = rep(NA, sum(!data$.drop)),
               ##
               event1 = event1,
@@ -1493,7 +1872,7 @@ netmetabin <- function(event1, n1, event2, n2,
               ##
               title = title,
               ##
-              data = data,
+              data = if (keepdata) data else NULL,
               data.design = dat.design,
               ##
               warn = warn,
@@ -1529,19 +1908,25 @@ netmetabin <- function(event1, n1, event2, n2,
   rownames(res$data) <- seq_len(nrow(res$data))
   
   
-  res$events.matrix <- netmatrix(res, event1 + event2, func = "sum")
-  ##
-  dat.e <- bySummary(c(event1, event2), c(treat1, treat2), long = FALSE)
-  rownames(dat.e) <- dat.e$indices
-  res$events.trts <- dat.e[trts, "sum"]
-  names(res$events.trts) <- trts
-  ##
   res$n.matrix <- netmatrix(res, n1 + n2, func = "sum")
   ##
-  dat.n <- bySummary(c(n1, n2), c(treat1, treat2), long = FALSE)
-  rownames(dat.n) <- dat.n$indices
-  res$n.trts <- dat.n[trts, "sum"]
+  dat.n <- data.frame(studlab = c(studlab, studlab),
+                      treat = c(treat1, treat2),
+                      n = c(n1, n2))
+  dat.n <- dat.n[!duplicated(dat.n[, c("studlab", "treat")]), ]
+  dat.n <- by(dat.n$n, dat.n$treat, sum, na.rm = TRUE)
+  res$n.trts <- as.vector(dat.n[trts])
   names(res$n.trts) <- trts
+  ##
+  res$events.matrix <- netmatrix(res, event1 + event2, func = "sum")
+  ##
+  dat.e <- data.frame(studlab = c(studlab, studlab),
+                      treat = c(treat1, treat2),
+                      n = c(event1, event2))
+  dat.e <- dat.e[!duplicated(dat.e[, c("studlab", "treat")]), ]
+  dat.e <- by(dat.e$n, dat.e$treat, sum, na.rm = TRUE)
+  res$events.trts <- as.vector(dat.e[trts])
+  names(res$events.trts) <- trts
   
   
   res
